/*******************************************************************************
**                         backupprofile.h
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  begin    : Mon Apr  8 22:52:08 CEST 2002
**  copyright: (C) 2002 by Florian Simnacher
**  email    : simnacher@gmx.de
**  $Id: backupprofile.h,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
*******************************************************************************/

/*******************************************************************************
**  This program is free software; you can redistribute it and/or             **
**  modify it under the terms of the GNU General Public License               **
**  as published by the Free Software Foundation; either version 2            **
**  of the License, or (at your option) any later version.                    **
**                                                                            **
**  This program is distributed in the hope that it will be useful,           **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of            **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
**  GNU General Public License for more details.                              **
**                                                                            **
**  You should have received a copy of the GNU General Public License         **
**  along with this program; if not, write to the Free Software               **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    **
**  MA  02111-1307, USA.                                                      **
*******************************************************************************/
#ifndef backupprofile_h
#define backupprofile_h

// Qt includes
#include <qdatetime.h>
#include <qstring.h>
#include <qtimer.h>
#include <qobject.h>

// KDE includes
#include <kurl.h>
#include <ktar.h>
#include <kio/job.h>

// My includes
#include "dumpable.h"
#include "timeinterval.h"

////////////////////////////////////////////////////////////////////////////////
///////                  class Backup                                    ///////
////////////////////////////////////////////////////////////////////////////////

class Backup: public QObject, public Dumpable
{
    Q_OBJECT

public:

    Backup( KURL surl, KURL aurl, bool readonly = false );
    ~Backup();

    void setSourceUrl( KURL url ) { mSourceUrl = url; }
    KURL getSourceUrl() const     { return mSourceUrl; }

    void setArchiveUrl( KURL url ) { mArchiveUrl = url; }
    KURL getArchiveUrl() const     { return mArchiveUrl; }

    void    doBackup();
    QString dumpInternal();

protected slots:
    void slotStartListing( KIO::Job* aJob );
    void slotGetEntriesAndPackEm( KIO::Job*, const KIO::UDSEntryList& list );
    void slotFinishedPacking( KIO::Job* aJob );
    void slotCopyArchive();
    void slotFinished( KIO::Job* aJob );

private:
    Backup( const Backup& b, bool readonly = true );
    KURL defaultArchiveName( KURL source, KURL archive );
    KURL dotCheck( KURL name );

private:
    KURL                mSourceUrl;
    KURL                mTmpSourceUrl;
    KURL                mArchiveUrl;
    KURL                mTmpArchiveUrl;
    bool                mbReadOnly;
    KArchive           *mpArchive;
};

////////////////////////////////////////////////////////////////////////////////
///////                  class BackupProfile                             ///////
////////////////////////////////////////////////////////////////////////////////

class BackupProfile : public QObject, public Dumpable
{

    Q_OBJECT

    friend class BackupProfileTest;

public:
    BackupProfile( Backup*      impl,
                   QString      identifier,
                   TimeInterval interv,
                   QDateTime    last,
                   bool         activ,
                   bool         readonly = false );
    ~BackupProfile();

    void activate();
    bool isActivated() const
        { return mbIsActivated; }

    void setSourceUrl( KURL url )
        { mpBackup->setSourceUrl( url ); }
    KURL getSourceUrl() const
        { return mpBackup->getSourceUrl(); }

    void setArchiveUrl( KURL url )
        { mpBackup->setArchiveUrl( url ); }
    KURL getArchiveUrl() const
        { return mpBackup->getArchiveUrl(); }

    void setIdentifier( QString id )
        { mIdentifier = id; }
    QString getIdentifier() const
        { return mIdentifier; }

    void setTimeBetweenBackups( TimeInterval t )
        { mTimeToBackup = t; }
    TimeInterval getTimeBetweenBackups() const
        { return mTimeToBackup; }

    void setLastTime( QDateTime q )
        { mLastBackupTime = q; }
    QDateTime getLastTime() const
        { return mLastBackupTime; }

    void restore();
    QString dumpInternal();

    static BackupProfile *newBackupProfile(
        KURL      source,
        KURL      backup,
        QString   identifier,
        int       time,
        QString   measurement,
        bool      activ,
        QDateTime last
        );

    static BackupProfile *newBackupProfileReadOnly(
        KURL      source,
        KURL      backup,
        QString   identifier,
        int       time,
        QString   measurement,
        bool      activ,
        QDateTime last
        );

    static BackupProfile *newBackupProfileReadOnly(
        const BackupProfile * const b
        );

public slots:
    void slotDoBackup();
    void slotDoOneTimeBackup();

private:
    BackupProfile( const BackupProfile& b );
    void operator=( const BackupProfile& );
    void setupTimer();
    void extractArchiveEntry( KArchiveEntry* entry, QString path );
    KURL getToRestoreBackup();
    bool restoreActionIsSave();

private:
    Backup            *mpBackup;
    QTimer            *mpTimer;

    QString            mIdentifier;
    TimeInterval       mTimeToBackup;
    QDateTime          mLastBackupTime;
    bool               mbIsActivated;
    bool               mReadOnly;
};

#endif
