/*******************************************************************************
**                         configurator.cpp
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  begin    : Son Apr 14 17:15:21 CEST 2002
**  copyright: (C) 2002 by Florian Simnacher
**  email    : simnacher@gmx.de
**  $Id: configurator.cpp,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
*******************************************************************************/

/*******************************************************************************
**  This program is free software; you can redistribute it and/or             **
**  modify it under the terms of the GNU General Public License               **
**  as published by the Free Software Foundation; either version 2            **
**  of the License, or (at your option) any later version.                    **
**                                                                            **
**  This program is distributed in the hope that it will be useful,           **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of            **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
**  GNU General Public License for more details.                              **
**                                                                            **
**  You should have received a copy of the GNU General Public License         **
**  along with this program; if not, write to the Free Software               **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    **
**  MA  02111-1307, USA.                                                      **
******************************************************************************/

// KDE includes
#include <kapplication.h>
#include <kmessagebox.h>
#include <klocale.h>

// My includes
#include "configurator.h"

static const QString CONFIG_VERSION = QString( "1" );


////////////////////////////////////////////////////////////////////////////////
///////	       class Configurator                                        ///////
////////////////////////////////////////////////////////////////////////////////

Configurator::Configurator()
    : Dumpable( "Configurator" ),
      mpTheConfig( kapp->config() )
{
    mpTheConfig->setGroup( "General" );
    mProfileNames      = mpTheConfig->readListEntry( "Profilenames" );
    mConfigFileVersion = mpTheConfig->readEntry( "Version", CONFIG_VERSION );

    // Check if all files listed in Profilenames (config file) are there
    QValueList<QString>::iterator it;
    for ( it = mProfileNames.begin(); *it; ++it )
        if ( ! mpTheConfig->hasGroup( *it ) )
        {
            KMessageBox::error(
                0L, i18n( "Your configuration file is corrupted" )
                );
            // Why the fuck does the following not terminate the
            // program (if uncommented of course)?
            // kapp->quit();
        }
}

Configurator::~Configurator()
{
    this->sync();
    mpTheConfig->sync();
}

void Configurator::inserted( BackupProfile *b )
{
    insertProfile( b );
    this->sync();
}
void Configurator::changed( BackupProfile *b )
{
    changeProfile( b );
    this->sync();
}
void Configurator::removed( QString s )
{
    removeProfile( s );
    this->sync();
}

void Configurator::doConfigure()
{
    QValueList<QString>::iterator it = mProfileNames.begin();

    while ( mProfileNames.end() != it )
    {
        ProfileMngrInterface::insert( nextProfile( *it ) );
        it++;
    }
}

void Configurator::sync()
{
    mpTheConfig->setGroup( "General" );
    mpTheConfig->writeEntry( "Profilenames", mProfileNames );
    mpTheConfig->writeEntry( "Version", CONFIG_VERSION );
    mpTheConfig->sync();
}


QString Configurator::dumpInternal()
{
    QString tmp;

    tmp.append( "mProfileNames: " );
    QValueList<QString>::iterator it;
    for ( it = mProfileNames.begin(); *it; ++it )
        tmp.append( *it + "," );
    tmp.append( "\n" );

    return tmp;
}

void Configurator::insertProfile( BackupProfile *b )
{
    if ( mpTheConfig->hasGroup( b->getIdentifier() ) )
        changeProfile( b );
    else
        newProfile( b );
}

void Configurator::newProfile( BackupProfile *b )
{
    if ( mpTheConfig->hasGroup( b->getIdentifier() ) )
        if ( KMessageBox::No == KMessageBox::questionYesNo(
                 0L, i18n( "The backupprofile: %1 already exists.\n"
                           "Shall I replace it in the config file?" ).arg(
                               b->getIdentifier()
                               ),
                 i18n( "Profile already exists" )
                 )
            )
        {
            return;
        }
        else
        {
            changeProfile( b );
        }
    mpTheConfig->setGroup( b->getIdentifier() );
    mpTheConfig->writeEntry( "SourceUrl",  b->getSourceUrl().url() );
    mpTheConfig->writeEntry( "ArchiveUrl", b->getArchiveUrl().url() );
    mpTheConfig->writeEntry(
        "TimeNumber", b->getTimeBetweenBackups().value()
        );
    mpTheConfig->writeEntry(
        "Measurement", b->getTimeBetweenBackups().measurement()
        );
    mpTheConfig->writeEntry( "LastBackup", b->getLastTime() );
    mpTheConfig->writeEntry( "isActivated", b->isActivated() );

    mProfileNames.append( b->getIdentifier() );
}

void Configurator::changeProfile( BackupProfile *b )
{
    Q_ASSERT( mpTheConfig->hasGroup( b->getIdentifier() ) );
    Q_ASSERT( 1 == mProfileNames.contains( b->getIdentifier() ) );

    mpTheConfig->deleteGroup( b->getIdentifier() );

    mpTheConfig->setGroup( b->getIdentifier() );
    mpTheConfig->writeEntry( "SourceUrl",  b->getSourceUrl().url() );
    mpTheConfig->writeEntry( "ArchiveUrl", b->getArchiveUrl().url() );
    mpTheConfig->writeEntry(
        "TimeNumber", b->getTimeBetweenBackups().value()
        );
    mpTheConfig->writeEntry(
        "Measurement", b->getTimeBetweenBackups().measurement()
        );
    mpTheConfig->writeEntry( "LastBackup", b->getLastTime() );
    mpTheConfig->writeEntry( "isActivated", b->isActivated() );
}

void Configurator::removeProfile( QString q )
{
    Q_ASSERT( 1 == mProfileNames.contains( q ) );
    mProfileNames.remove( q );

    Q_ASSERT( mpTheConfig->hasGroup( q ) );
    mpTheConfig->deleteGroup( q );
}

BackupProfile *Configurator::nextProfile( QString aProfileName )
{
    mpTheConfig->setGroup( aProfileName );

    KURL source( mpTheConfig->readEntry( "SourceUrl" ));
    KURL archive( mpTheConfig->readEntry( "ArchiveUrl" ) );

    QString          id = aProfileName;
    int            time = mpTheConfig->readNumEntry( "TimeNumber" );
    QString measurement = mpTheConfig->readEntry( "Measurement" );
    bool          activ = mpTheConfig->readBoolEntry( "isActivated" );
    QDateTime         q = mpTheConfig->readDateTimeEntry( "LastBackup" );

    BackupProfile *tmp =  BackupProfile::newBackupProfile(
        source, archive, aProfileName, time, measurement, activ, q
        );

    return tmp;
}
