/*******************************************************************************
**                        profilemanager.cpp
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  begin    : Mit Apr 10 15:30:47 CEST 2002
**  copyright: (C) 2002 by Florian Simnacher
**  email    : simnacher@gmx.de
**  $Id: profilemanager.cpp,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
*******************************************************************************/

/*******************************************************************************
**  This program is free software; you can redistribute it and/or             **
**  modify it under the terms of the GNU General Public License               **
**  as published by the Free Software Foundation; either version 2            **
**  of the License, or (at your option) any later version.                    **
**                                                                            **
**  This program is distributed in the hope that it will be useful,           **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of            **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
**  GNU General Public License for more details.                              **
**                                                                            **
**  You should have received a copy of the GNU General Public License         **
**  along with this program; if not, write to the Free Software               **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    **
**  MA  02111-1307, USA.                                                      **
*******************************************************************************/

// Qt includes
#include <qdatetime.h>

// KDE includes
#include <kmessagebox.h>
#include <klocale.h>

// My includes
#include "profilemanager.h"

////////////////////////////////////////////////////////////////////////////////
///////               class ProfileManager                               ///////
////////////////////////////////////////////////////////////////////////////////

ProfileManager::ProfileManager()
    : Dumpable( "ProfileManager" )
{
    mProfileList.setAutoDelete( true );
}

ProfileManager::~ProfileManager()
{
    // EVIL DIRTY UGLY HACK that shows that we do not have the right design
    BackupProfile *b;
    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        Subject::_evilHack_SaveBackupProfile( b );
    }
}

void ProfileManager::insertCalled( BackupProfile *b )
{
    insertProfile( b );
}

void ProfileManager::changeCalled( BackupProfile *b )
{
    changeProfile( b );
}
void ProfileManager::removeCalled( QString s )
{
    removeProfile( s );
}

bool ProfileManager::existsCalled( QString s )
{
    return 0L != getProfile( s );
}

void ProfileManager::restoreCalled( QString s )
{
    getProfile( s )->restore();
}

QString ProfileManager::dumpInternal()
{
    BackupProfile *b;
    QString        tmp;

    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        tmp += b->dump();
    }

    return tmp;
}

void ProfileManager::activate()
{
    BackupProfile *b;
    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        b->activate();
    }
}

void ProfileManager::slotDoBackup( QString identifier )
{
    Q_ASSERT( existsCalled( identifier )  );

    getProfile( identifier )->slotDoOneTimeBackup();
}

void ProfileManager::insertProfile( BackupProfile *bp )
{
    if ( 0L != getProfile( bp->getIdentifier() ) )
        changeProfile( bp );
    else
        newProfile( bp );

    bp->activate();
    return;
}

void ProfileManager::newProfile( BackupProfile *bp )
{
    Q_ASSERT( 0L == getProfile( bp->getIdentifier() ) );

    Q_CHECK_PTR( bp );
    mProfileList.append( bp );
}

void ProfileManager::changeProfile( BackupProfile *bp )
{
    Q_ASSERT( 0L != getProfile( bp->getIdentifier() ));

    int index = mProfileList.find( getProfile( bp->getIdentifier() ) );
    mProfileList.remove( index );
    mProfileList.insert( index, bp );
}

void ProfileManager::removeProfile( QString id )
{
    const BackupProfile * const b = getProfile( id );
    if ( 0L == (long int) b )
    {
        KMessageBox::error(
            0L,
            i18n( "Sorry, could not remove profile: %1.\n"
                  "The Identifier is not in the list" ).arg( id )
            );
        return;
    }

    int index = mProfileList.find( b );
    Q_ASSERT( -1 != index );

    if ( !mProfileList.remove( index ) )
        Q_ASSERT( false );
}

BackupProfile* ProfileManager::getProfile( QString id )
{
    BackupProfile *b      = 0L;
    BackupProfile *result = 0L;

    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        if ( id == b->getIdentifier() )
        {
            if ( 0L != result )
                Q_ASSERT( false ); // Consistency check, no Identifier twice
            result = b;
        }
    }
    return result;
}
