/***************************************************************************
**                       backupprofiletest.cpp
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  begin    : Mit Jul 10 22:42:21 CEST 2002
**  version  : $Id: backupprofiletest.cpp,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
**  copyright: (C) 2002 by Florian Simnacher
**  email    : simnacher@gmx.de
***************************************************************************/

/***************************************************************************
**  This program is free software; you can redistribute it and/or         **
**  modify it under the terms of the GNU General Public License           **
**  as published by the Free Software Foundation; either version 2        **
**  of the License, or (at your option) any later version.                **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program; if not, write to the Free Software           **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                **
**  MA  02111-1307, USA.                                                  **
***************************************************************************/
// Standard Includes
#include <iostream>
#include <unistd.h>

// Test Includes
#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestResult.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>

// Qt Includes
#include <qdatetime.h>

// KDE Includes
#include <kinstance.h>
#include <kdebug.h>
#include <ktar.h>
#include <kprocess.h>

// My Includes
#include "backupprofile.h"
#include "timeinterval.h"
#include "testenv.h"

class BackupProfileTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( BackupProfileTest );
    CPPUNIT_TEST( test_Initialization );
    CPPUNIT_TEST( test_restore_archive_file );
    CPPUNIT_TEST( test_restore_archive_dir );
    CPPUNIT_TEST( test_extractArchivEntry );
    CPPUNIT_TEST( test_getToRestoreBackup );
    CPPUNIT_TEST_SUITE_END();

public:
    void test_Initialization()
        {
            ArchiveCreatorBase *archiver = new TarArchiveCreator;
            Backup *backup  = new Backup(
                "file:/tmp/konserve-testdir", "file:/tmp/", archiver
                );
            QDateTime q = QDateTime::currentDateTime();

            BackupProfile b(
                backup,
                "foofoo",
                *( new TimeInterval( 10, TimeInterval::TIME_SEC ) ),
                q,
                true
                );
            cout << "Selftesting not implemented" << endl;
        }
    void test_restore_archive_file()
        {
            BackupProfile *profile = BackupProfile::newBackupProfile(
                KURL( "file:/tmp/konserve-testdir" ),
                KURL( "file:/tmp/out.tar.gz" ),
                "test_restore",
                30,
                "min",
                false,
                QDateTime::currentDateTime()
                );
            profile->slotDoOneTimeBackup();
            mpTestEnv->removeFile( "file:/tmp/konserve-testdir" );
            profile->restore();

            CPPUNIT_ASSERT( mpTestEnv->fileExists( "file:/tmp/konserve-testdir/foo" ) );
            CPPUNIT_ASSERT( mpTestEnv->fileExists( "file:/tmp/konserve-testdir/subdir/foo" ) );
        }
    void test_restore_archive_dir()
        {
            BackupProfile *profile = BackupProfile::newBackupProfile(
                KURL( "file:/tmp/konserve-testdir" ),
                KURL( "file:/tmp/" ),
                "test_restore",
                30,
                "min",
                false,
                QDateTime::currentDateTime()
                );
            profile->slotDoOneTimeBackup();
            mpTestEnv->removeFile( "file:/tmp/konserve-testdir" );
            profile->restore();

            CPPUNIT_ASSERT( mpTestEnv->fileExists( "file:/tmp/konserve-testdir/foo" ) );
            CPPUNIT_ASSERT( mpTestEnv->fileExists( "file:/tmp/konserve-testdir/subdir/foo" ) );
        }
    void test_extractArchivEntry() // XXX To be tested
        {
            BackupProfile *profile = BackupProfile::newBackupProfile(
                KURL( "file:/tmp/konserve-testdir" ),
                KURL( "file:/tmp/" ),
                "test_restore",
                30,
                "min",
                false,
                QDateTime::currentDateTime()
                );
           profile->slotDoOneTimeBackup();
           KArchive *archive = new KTar(
                "/tmp/konserve-testdir-20020905103000.tar.gz"
                );
            mpTestEnv->removeFile( "file:/tmp/konserve-testdir" );

            if ( ! archive->open( IO_ReadOnly ) )
            {
                kdDebug(0) << "Shit happend. Couldn't open archive" << endl;
                ::exit( -1 );
            }
            KArchiveEntry *root = const_cast<KArchiveDirectory*>( archive->directory() );
            profile->extractArchiveEntry( root, QString::null );
            archive->close();
            CPPUNIT_ASSERT( mpTestEnv->fileExists( "file:/tmp/konserve-testdir/foo" ) );
            CPPUNIT_ASSERT( mpTestEnv->fileExists( "file:/tmp/konserve-testdir/subdir/foo" ) );
        }
    void test_getToRestoreBackup()
        {
            BackupProfile *profile = BackupProfile::newBackupProfile(
                KURL( "file:/tmp/konserve" ),
                KURL( "file:/tmp/konserve-testdir/" ),
                "test_restore",
                30,
                "min",
                false,
                QDateTime::currentDateTime()
                );
            QString expected( "/tmp/konserve-testdir/konserve-20020801112430.tar.gz" );

            CPPUNIT_ASSERT( expected == profile->getToRestoreBackup().path() );
        }
    void setUp()
        {
            mpTestEnv = new TestEnvironment;
            mpTestEnv->setupTestEnvironment();
        }
    void tearDown()
        {
            mpTestEnv->removeFile(
                "file:/tmp/konserve-testdir-20020905103000.tar.gz"
                );
            mpTestEnv->removeFile(
                "file:/tmp/konserve-testdir"
                );
            delete mpTestEnv;
            mpTestEnv = 0L;
        }
private:
    TestEnvironment *mpTestEnv;
};

int main( /* int argc, char** argv */ )
{
    KInstance myInstance( "BackupProfileTest" );

    CppUnit::TextUi::TestRunner runner;
    runner.addTest( BackupProfileTest::suite() );
    runner.run();

    return 0;
}
