/***************************************************************************
                        profilemanagertest.cpp
			 -------------------
    begin    : Mit Apr 10 00:20:32 CEST 2002
    version  : $Id: profilemanagertest.cpp,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
    copyright: (C) 2002 by Florian Simnacher
    email    : simnacher@gmx.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>

#include <kapp.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <qstringlist.h>
#include <qdatetime.h>
#include <qfile.h>
#include <dcopclient.h>

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestResult.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>

#include "profilemanager.h"
#include "timeinterval.h"

static DCOPClient     *DCOPClient;

static void rebuildSycoca()
{
    if ( ! DCOPClient->send(
             "kded",
             "kbuildsycoca",
             "recreate()",
             QString::null
             )
        )
    {
        cerr << "XXX: Send failed" << endl;
        assert( false );
    }
}


class ProfileManagerTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( ProfileManagerTest );
    CPPUNIT_TEST( testNewProfile );
    CPPUNIT_TEST( testRemoveProfile );
    CPPUNIT_TEST( testInsertProfile );
    CPPUNIT_TEST_SUITE_END();
public:
    ProfileManagerTest()
        {
            mpFileName = new QString (
                QString( ::getenv( "HOME" ) ) + QString( "/.kde/share/config/profilemanagertestrc" )
                );

            fooFile = new QString("[General]\n"
                                  "Profilenames=foo-file\n"
                                  "Version=1\n"
                                  "\n"
                                  "[foo-file]\n"
                                  "ArchiveUrl=file:/tmp/\n"
                                  "LastBackup=2002,9,5,10,30,0\n"
                                  "Measurement=sec\n"
                                  "SourceUrl=file:/home/florian/tmp/\n"
                                  "TimeNumber=10\n"
                                  "isActivated=false\n"
                );
            barFile = new QString("[General]\n"
                                  "Profilenames=foo-file,bar-file\n"
                                  "Version=1\n"
                                  "\n"
                                  "[bar-file]\n"
                                  "ArchiveUrl=file:/tmp/\n"
                                  "LastBackup=2002,9,5,10,30,0\n"
                                  "Measurement=hour\n"
                                  "SourceUrl=file:/home/florian/\n"
                                  "TimeNumber=20\n"
                                  "isActivated=false\n"
                                  "\n"
                                  "[foo-file]\n"
                                  "ArchiveUrl=file:/tmp/\n"
                                  "LastBackup=2002,9,5,10,30,0\n"
                                  "Measurement=sec\n"
                                  "SourceUrl=file:/home/florian/tmp/\n"
                                  "TimeNumber=10\n"
                                  "isActivated=false\n"
                );

        }
    void setUp()
        {
            ::system( "rm $HOME/.kde/share/config/profilemanagertestrc" );
            rebuildSycoca(); // FUCK: This one works sometimes, sometimes not.
            ::sleep( 1 ); // Perhaps Sycoca is not fast enough?

            pm = new ProfileManager;
        }
    void tearDown()
        {
            delete pm;
        }
    void testNewProfile()
        {
//             ArchiveCreatorBase *archiver = new TarArchiveCreator;
//             BackupProfileImpl  *impl_1 = new BackupProfileImpl(
//                 "file:/home/florian/tmp/", "file:/tmp/", archiver
//                 );
//             pm->newProfile(
//                 new BackupProfile(
//                     impl_1, "foo-file", * (new TimeInterval( 10, TimeInterval::TIME_SEC )),
//                     QDateTime::currentDateTime()
//                     )
//                 );
//             CPPUNIT_ASSERT( testComp( *fooFile, *mpFileName ) );
        }
    void testRemoveProfile()
        {
            QFile *file = new QFile( *mpFileName );
            file->open( IO_WriteOnly );
            QTextStream out( file );
            out << *fooFile;
            file->close();
            rebuildSycoca();

            QString expected( "[General]\n"
                              "Profilenames=\n"
                              "Version=1\n" );

            CPPUNIT_ASSERT( testComp( *fooFile, *mpFileName ) );

//             pm->removeProfile( "foo-file" );

            CPPUNIT_ASSERT( testComp( expected, *mpFileName ) );
        }
    void testInsertProfile()
        {
            // XXX GO_ON_HERE
//             OLD
//             ArchiveCreatorBase *archiver = new TarArchiveCreator;
//             BackupProfileImpl  *impl_1 = new BackupProfileImpl(
//                 "file:/home/florian/tmp/", "file:/tmp/", archiver
//                 );
//             pm->insertProfile(
//                 new BackupProfile(
//                     impl_1, "foo-file", * (new TimeInterval( 10, TimeInterval::TIME_SEC )),
//                     QDateTime::currentDateTime()
//                     )
//                 );
//             CPPUNIT_ASSERT( testComp( *fooFile, *mpFileName ) );

//             ArchiveCreatorBase *archiver_2 = new TarArchiveCreator;
//             BackupProfileImpl  *impl_2 = new BackupProfileImpl(
//                 "file:/home/florian/", "file:/tmp/", archiver_2
//                 );
//             pm->insertProfile(
//                 new BackupProfile(
//                     impl_2, "bar-file", * (new TimeInterval( 20, TimeInterval::TIME_HOUR )),
//                     QDateTime::currentDateTime()
//                     )
//                 );
//             CPPUNIT_ASSERT( testComp( *barFile, *mpFileName ) );
        }
private:
    bool testComp( QString original, QString filename )
        {
            QFile file( filename );
            if ( !file.open( IO_ReadOnly ))
            {
                cerr << "No file: " << filename << endl;
                ::exit(-1);
            }
            QString fileString( file.readAll() );
            file.close();

//            cerr << "XXX: " << endl << original << endl << fileString << endl;

            return (original == fileString);
        }
    QString        *mpFileName;
    ProfileManager *pm;
    QString        *fooFile;
    QString        *barFile;
};


int main( int argc, char** argv )
{
    KAboutData kad( "profilemanagertest",
		    "profilemanagertest",
		    "0.0.1" );

    KCmdLineArgs::init( argc, argv, &kad );
    KApplication mykapp;

    DCOPClient = kapp->dcopClient();
    if ( ! DCOPClient->attach() )
        cerr << "XXX: dcop did not work" << endl;

    CppUnit::TextUi::TestRunner runner;
    runner.addTest( ProfileManagerTest::suite() );
    runner.run();
    mykapp.quit();

    return 0;
}

