/***************************************************************************
**                      profilemngrifacetest.cpp
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  begin    : Mit Jun 19 10:41:24 CEST 2002
**  version  : $Id: profilemngrifacetest.cpp,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
**  copyright: (C) 2002 by Florian Simnacher
**  email    : simnacher@gmx.de
***************************************************************************/

/***************************************************************************
**  This program is free software; you can redistribute it and/or         **
**  modify it under the terms of the GNU General Public License           **
**  as published by the Free Software Foundation; either version 2        **
**  of the License, or (at your option) any later version.                **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program; if not, write to the Free Software           **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                **
**  MA  02111-1307, USA.                                                  **
***************************************************************************/
#include <iostream>
#include <fstream>

#include <kinstance.h>
#include <kdebug.h>

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestResult.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/ui/text/TestRunner.h>

#include <profilemngrinterface.h>
#include <qfile.h>

class Foo : public ProfileMngrInterface
{
public:
    virtual void inserted( BackupProfile * )
        {
            std::cerr << "Foo inserted" << endl;
        }
    virtual void changed( BackupProfile )
        {
            std::cerr << "Foo changed" << endl;
        }
    virtual void removed( QString )
        {
            std::cerr << "Foo removed" << endl;
        }
    void checkInsert()
        {
            _IO_ostream_withassign *cerr_backup = new _IO_ostream_withassign();
            std::ofstream *out = new std::ofstream( "checkInsert.txt" );

            *cerr_backup = std::cerr;
            if ( out->good() ) std::cerr = *out;

            insert(
                BackupProfile::newBackupProfile(
                    KURL( "file:/home/florian/" ),
                    KURL( "file:/tmp/" ),
                    QString( "identifier" ),
                    10,
                    QString( "sec" ),
                    false,
                    QDateTime::currentDateTime()
                    )
                );
            std::cerr = *cerr_backup;
            out->close();

            QString expected( "TheSubject existsCalled\n"
                              "TheSubject insertCalled\n"
                              "Bar inserted\n" );
            CPPUNIT_ASSERT( testComp( expected, "checkInsert.txt" ) );
        }
    void checkChange()
        {
            _IO_ostream_withassign *cerr_backup = new _IO_ostream_withassign();
            std::ofstream *out = new std::ofstream( "checkChange.txt" );

            *cerr_backup = std::cerr;
            if ( out->good() ) std::cerr = *out;

            change(
                BackupProfile::newBackupProfile(
                    KURL( "file:/home/florian/" ),
                    KURL( "file:/tmp/" ),
                    QString( "identifier" ),
                    10,
                    QString( "sec" ),
                    false,
                    QDateTime::currentDateTime()
                    )
                );
            std::cerr = *cerr_backup;
            out->close();

            QString expected( "TheSubject existsCalled\n"
                              "TheSubject changeCalled\n"
                              "Bar changed\n" );
            CPPUNIT_ASSERT( testComp( expected, "checkChange.txt" ) );
        }
    void checkRemove()
        {
            _IO_ostream_withassign *cerr_backup = new _IO_ostream_withassign();
            std::ofstream *out = new std::ofstream( "checkRemove.txt" );

            *cerr_backup = std::cerr;
            if ( out->good() ) std::cerr = *out;

            remove( QString( "foobarbaz" ) );
            std::cerr = *cerr_backup;
            out->close();

            QString expected( "TheSubject existsCalled\n"
                              "TheSubject removeCalled\n"
                              "Bar removed\n" );
            CPPUNIT_ASSERT( testComp( expected, "checkRemove.txt" ) );
        }
    void checkRestore()
        {
            _IO_ostream_withassign *cerr_backup = new _IO_ostream_withassign();
            std::ofstream *out = new std::ofstream( "checkRestore.txt" );

            *cerr_backup = std::cerr;
            if ( out->good() ) std::cerr = *out;

            restore( "mybackup" );

            std::cerr = *cerr_backup;
            out->close();

            QString expected( "TheSubject existsCalled\n"
                              "TheSubject restoreCalled\n" );
            CPPUNIT_ASSERT( testComp( expected, "checkRestore.txt" ) );
        }
private:
    bool testComp( QString original, QString filename )
        {
            QFile file( filename );
            if ( !file.open( IO_ReadOnly ))
            {
                cerr << "No file: " << filename << endl;
                ::exit(-1);
            }
            QString fileString( file.readAll() );
            file.close();

//            cerr << "XXX: " << endl << original << endl << fileString << endl;

            return (original == fileString);
        }
};

class Bar : public ProfileMngrInterface
{
public:
    virtual void inserted( BackupProfile * )
        {
            std::cerr << "Bar inserted" << endl;
        }
    virtual void changed( BackupProfile * )
        {
            std::cerr << "Bar changed" << endl;
        }
    virtual void removed( QString )
        {
            std::cerr << "Bar removed" << endl;
        }
};

class TheSubject : public Subject
{
public:
    TheSubject() : mbExists( true ) {
        // NOOP
    }
    virtual void insertCalled( BackupProfile * )
        {
            std::cerr << "TheSubject insertCalled" << endl;
        }
    virtual void changeCalled( BackupProfile * )
        {
            std::cerr << "TheSubject changeCalled" << endl;
        }
    virtual void removeCalled( QString )
        {
            std::cerr << "TheSubject removeCalled" << endl;
        }
    virtual bool existsCalled( QString )
        {
            std::cerr << "TheSubject existsCalled" << endl;
            return mbExists;
        }
    virtual void restoreCalled( QString )
        {
            std::cerr << "TheSubject restoreCalled" << endl;
        }
    void setExits( bool val )
        {
            mbExists = val;
        }
private:
    bool mbExists;
};

class ProfileMngrIfaceTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE( ProfileMngrIfaceTest );
    CPPUNIT_TEST( test_Insert );
    CPPUNIT_TEST( test_Change );
    CPPUNIT_TEST( test_Remove );
    CPPUNIT_TEST( test_Restore );
    CPPUNIT_TEST_SUITE_END();

public:
    void setUp()
        {
            mpFoo     = new Foo;
            mpBar     = new Bar;
            mpSubject = new TheSubject;

            mpSubject->addObserver( mpFoo );
            mpSubject->addObserver( mpBar );
        }
    void tearDown()
        {
            mpSubject->removeObserver( mpFoo );
            mpSubject->removeObserver( mpBar );

            delete mpFoo;
            delete mpBar;
            delete mpSubject;
        }
    void test_Insert()
        {
            (dynamic_cast<TheSubject*>(mpSubject))->setExits( false );
            mpFoo->checkInsert();
        }
    void test_Change()
        {
            mpFoo->checkChange();
        }
    void test_Remove()
        {
            mpFoo->checkRemove();
        }
    void test_Restore()
        {
            mpFoo->checkRestore();
        }

private:
    Foo     *mpFoo;
    Bar     *mpBar;
    Subject *mpSubject;
};

int main( /* int argc, char** argv */ )
{
    KInstance myInstance( "profilemngrifacetest" );

    CppUnit::TextUi::TestRunner runner;
    runner.addTest( ProfileMngrIfaceTest::suite() );
    runner.run();
    return 0;
}



