/***************************************************************************
**                              testenv.cpp
**                                part of
**                konserve -- A small backup application
**			 -------------------
**  begin    : Fre Jul 12 18:11:31 CEST 2002
**  version  : $Id: testenv.cpp,v 1.1.1.1 2003/09/17 07:52:06 florian Exp $
**  copyright: (C) 2002 by Florian Simnacher
**  email    : simnacher@gmx.de
***************************************************************************/

/***************************************************************************
**  This program is free software; you can redistribute it and/or         **
**  modify it under the terms of the GNU General Public License           **
**  as published by the Free Software Foundation; either version 2        **
**  of the License, or (at your option) any later version.                **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program; if not, write to the Free Software           **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                **
**  MA  02111-1307, USA.                                                  **
***************************************************************************/

// Standard Includes
#include <unistd.h>

// Qt Includes
#include <qfile.h>
#include <qtextstream.h>
#include <qfileinfo.h>

// KDE Includes
#include <kprocess.h>
#include <kdebug.h>

// My includes
#include "testenv.h"

const char* FILE_ONE   = "This is the content of file one";
const char* FILE_TWO   = "This is the content of file two";
const char* FILE_THREE = "This is the content of file three";

////////////////////////////////////////////////////////////////////////////
////////                 class TestEnvironment                    //////////
////////////////////////////////////////////////////////////////////////////

TestEnvironment::TestEnvironment()
    : mRootName( "file:/tmp/konserve-testdir" )
{
    // NOOP
}

TestEnvironment::~TestEnvironment()
{
//    removeFile( mRootName );
}

TestEnvironment::TestEnvironment( const TestEnvironment& )
{
    // NOOP
}

void TestEnvironment::setupTestEnvironment()
{
    if ( fileExists( mRootName ) ) {
        removeFile( mRootName );
    }

    createFile( mRootName );
    createFile( mRootName.path() + "/bar", FILE_ONE );
    createFile( mRootName.path() + "/foo", FILE_TWO );
    createFile( mRootName.path() + "/bug", FILE_THREE);

    createFile( mRootName.path() + "/subdir" );
    createFile( mRootName.path() + "/subdir/bar", FILE_ONE );
    createFile( mRootName.path() + "/subdir/foo", FILE_TWO );
    createFile( mRootName.path() + "/subdir/bug", FILE_THREE);

    createFile( mRootName.path() + "/konserve-20020801112230.tar.gz", FILE_THREE);
    createFile( mRootName.path() + "/konserve-20020801112330.tar.gz", FILE_THREE);
    createFile( mRootName.path() + "/konserve-20020801112430.tar.gz", FILE_THREE);

    return;
}

bool TestEnvironment::fileExists( KURL file )
{
    QFileInfo info( file.path() );
    return info.exists();
}

void TestEnvironment::removeFile( KURL file )
{
    if ( ! file.path().startsWith( "/tmp" ) )
        ::exit( -1 );

    KProcess process;
    process << "rm";
    process << "-f" << "-r" << file.path();
    process.setUseShell( true );
    process.start( KProcess::Block );

    Q_ASSERT( ! fileExists( file ) );
}

void TestEnvironment::createFile( KURL url, QString content )
{
    if ( QString::null == content )
    {
        KProcess process;
        process << "mkdir";
        process << url.path();
        process.setUseShell( true );
        process.start( KProcess::Block );

        Q_ASSERT( fileExists( url ) );

        return;
    }
    QFile f( url.path() );
    if ( !f.open( IO_WriteOnly ) )
    {
        kdDebug() << "Shit happend" << endl;
        ::exit(-1);
    }
    QTextStream out( &f );
    out << content;
    f.close();
    return;
}

void TestEnvironment::waitFor( int secs )
{
    ::sleep( secs );
}

// int main()
// {

//     TestEnvironment env;
//     env.setupTestEnvironment();
//     return 0;
// }

